#include "lib.h"
#include "array.h"
#include "mail-storage.h"
#include "mail-storage-private.h"
#include "mail-namespace.h"
#include "readonly-plugin.h"
#include <stdlib.h>

const char *readonly_plugin_version = PACKAGE_VERSION;

void (*readonly_next_hook_mail_storage_created)(struct mail_storage *storage);

static MODULE_CONTEXT_DEFINE_INIT(readonly_storage_module, &mail_storage_module_register);

struct readonly_mailbox {
	/* must be called module_ctx */
	union mailbox_module_context module_ctx;
};

/*
Override the default mailbox_create() function.
All we do is return with a failure. We say NO_PERMISSION
*/
static int
readonly_mailbox_create(struct mail_storage *storage, const char *name, bool directory)
{
	(void) name;
	(void) directory;

	/*
	// Just set an error message (No Permissions) and fail
	*/

	mail_storage_set_error(storage, MAIL_ERROR_PERM, MAIL_ERRSTR_NO_PERMISSION);

	return -1;

}

/*
Override the default mailbox_open() function.
We set the 'read only' flag and flags that should
stop dovecot from writing to disk.
*/
static struct mailbox *
readonly_mailbox_open(struct mail_storage *storage, const char *name,
	struct istream *input, enum mailbox_open_flags flags)
{
	union mail_storage_module_context *rostorage = READONLY_CONTEXT(storage);
	struct mailbox *box;

	/* Testing debug */
	i_info("Looks like %s is opened.", name);
	if((flags & MAILBOX_OPEN_SAVEONLY) == MAILBOX_OPEN_SAVEONLY)
	{
		i_info("Looks like MAILBOX_OPEN_SAVEONLY is set.");
	}

	/*
	// Reset all flags?
	*/
	flags = 0;

	/* Unset this, not needed if flags reset above? */
	// flags &= ~MAILBOX_OPEN_SAVEONLY;
	/* Mailbox must not be modified even if asked */
	flags |= MAILBOX_OPEN_READONLY;
	/* Don't reset MAIL_RECENT flags when syncing */
	flags |= MAILBOX_OPEN_KEEP_RECENT;
	/* Don't create index files for the mailbox */
	flags |= MAILBOX_OPEN_NO_INDEX_FILES;

	box = rostorage->super.mailbox_open(storage, name, input, flags);
	return box;
}

/*
Hook function.
Set our context and override some mailbox functions.
*/
static void readonly_mail_storage_created(struct mail_storage *storage)
{
	union mail_storage_module_context *rostorage;

	rostorage = p_new(storage->pool, union mail_storage_module_context, 1);
	rostorage->super = storage->v;
	storage->v.mailbox_open = readonly_mailbox_open;
	storage->v.mailbox_create = readonly_mailbox_create;

	MODULE_CONTEXT_SET_SELF(storage, readonly_storage_module, rostorage);

	if(readonly_next_hook_mail_storage_created != NULL)
		readonly_next_hook_mail_storage_created(storage);
}

void readonly_plugin_init(void)
{
	readonly_next_hook_mail_storage_created = hook_mail_storage_created;
	hook_mail_storage_created = readonly_mail_storage_created;
}

void readonly_plugin_deinit(void)
{
	hook_mail_storage_created = readonly_next_hook_mail_storage_created;
}
